@@ -49,6 +49,9 @@
 /* close all FDs >= a specified value */
 extern void closeall(int fd);
 
+/* Close a specific file descriptor and replace it with -1 */
+extern void fd_close(int *fd);
+
 void fd_set_close_on_exec(int fd);
 /*
  *  Sets the file descriptor (fd) to be closed on exec().
@@ -95,6 +98,9 @@ pid_t fd_is_read_lock_blocked(int fd);
  *    returns the pid of the process holding the lock; o/w, returns 0.
  */
 
+/* return true if fd is writable right now */
+extern bool fd_is_writable(int fd);
+
 extern int wait_fd_readable(int fd, int time_limit);
 /* Wait for a file descriptor to be readable (up to time_limit seconds).
  * Return 0 when readable or -1 on error */
@@ -181,4 +187,15 @@ extern int mkdirpath(const char *pathname, mode_t mode, bool is_dir);
  */
 extern int rmdir_recursive(const char *path, bool remove_top);
 
+/*
+ * Use ioctl(FIONREAD) to get number of bytes in buffer waiting for read().
+ * IN fd - file descriptor to query
+ * IN/OUT readable_ptr - Pointer to populate if ioctl() is able to query
+ *	successfully. Only changed if RET=SLURM_SUCCESS.
+ * IN con_name - descriptive name for fd connection (for logging)
+ * RET SLURM_SUCCESS or error
+ */
+extern int fd_get_readable_bytes(int fd, int *readable_ptr,
+				 const char *con_name);
+
 #endif /* !_FD_H */
