@@ -3,7 +3,7 @@
  *****************************************************************************
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  Copyright (C) 2008-2010 Lawrence Livermore National Security.
- *  Copyright (C) 2010-2015 SchedMD LLC.
+ *  Copyright (C) SchedMD LLC.
  *  Copyright (C) 2013      Intel, Inc.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Kevin Tew <tew1@llnl.gov>, et. al.
@@ -56,26 +56,25 @@
 #include <time.h>
 #include <unistd.h>
 
-/* PROJECT INCLUDES */
 #include "src/common/assoc_mgr.h"
 #include "src/common/fd.h"
 #include "src/common/forward.h"
-#include "src/interfaces/hash.h"
 #include "src/common/log.h"
 #include "src/common/macros.h"
 #include "src/common/net.h"
 #include "src/common/pack.h"
 #include "src/common/read_config.h"
-#include "src/interfaces/accounting_storage.h"
-#include "src/interfaces/auth.h"
-#include "src/common/slurm_protocol_interface.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_protocol_common.h"
 #include "src/common/slurm_protocol_pack.h"
+#include "src/common/slurm_protocol_socket.h"
 #include "src/common/strlcpy.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
+#include "src/interfaces/accounting_storage.h"
+#include "src/interfaces/auth.h"
+#include "src/interfaces/hash.h"
 #include "src/interfaces/topology.h"
 
 #include "src/slurmdbd/read_config.h"
@@ -423,7 +422,7 @@ extern uint16_t slurm_get_track_wckey(void)
 		track_wckey = slurmdbd_conf->track_wckey;
 	} else {
 		conf = slurm_conf_lock();
-		track_wckey = conf->conf_flags & CTL_CONF_WCKEY ? 1 : 0;
+		track_wckey = conf->conf_flags & CONF_FLAG_WCKEY ? 1 : 0;
 		slurm_conf_unlock();
 	}
 	return track_wckey;
@@ -563,39 +562,6 @@ char *slurm_get_acct_gather_interconnect_type(void)
 	return acct_gather_interconnect_type;
 }
 
-/* slurm_get_ext_sensors_type
- * get ExtSensorsType from slurm_conf object
- * RET char *   - ext_sensors type, MUST be xfreed by caller
- */
-char *slurm_get_ext_sensors_type(void)
-{
-	char *ext_sensors_type = NULL;
-	slurm_conf_t *conf;
-
-	if (slurmdbd_conf) {
-	} else {
-		conf = slurm_conf_lock();
-		ext_sensors_type =
-			xstrdup(conf->ext_sensors_type);
-		slurm_conf_unlock();
-	}
-	return ext_sensors_type;
-}
-
-extern uint16_t slurm_get_ext_sensors_freq(void)
-{
-	uint16_t freq = 0;
-	slurm_conf_t *conf;
-
-	if (slurmdbd_conf) {
-	} else {
-		conf = slurm_conf_lock();
-		freq = conf->ext_sensors_freq;
-		slurm_conf_unlock();
-	}
-	return freq;
-}
-
 /*
  * returns the configured GpuFreqDef value
  * RET char *    - GpuFreqDef value,  MUST be xfreed by caller
@@ -650,20 +616,6 @@ char *slurm_get_select_type(void)
 	return select_type;
 }
 
-/** Return true if (remote) system runs Cray Aries */
-bool is_cray_select_type(void)
-{
-	bool result = false;
-
-	if (slurmdbd_conf) {
-	} else {
-		slurm_conf_t *conf = slurm_conf_lock();
-		result = !xstrcasecmp(conf->select_type, "select/cray_aries");
-		slurm_conf_unlock();
-	}
-	return result;
-}
-
 /*  slurm_get_srun_port_range()
  */
 uint16_t *
@@ -681,19 +633,6 @@ slurm_get_srun_port_range(void)
 	return ports;	/* CLANG false positive */
 }
 
-/* slurm_get_core_spec_plugin
- * RET core_spec plugin name, must be xfreed by caller */
-char *slurm_get_core_spec_plugin(void)
-{
-	char *core_spec_plugin = NULL;
-	slurm_conf_t *conf;
-
-	conf = slurm_conf_lock();
-	core_spec_plugin = xstrdup(conf->core_spec_plugin);
-	slurm_conf_unlock();
-	return core_spec_plugin;
-}
-
 /* Change general slurm communication errors to slurmctld specific errors */
 static void _remap_slurmctld_errno(void)
 {
@@ -784,7 +723,6 @@ static int _open_controller(slurm_addr_t *addr, int *index,
 {
 	int fd = -1;
 	slurm_protocol_config_t *proto_conf = NULL;
-	int retry, max_retry_period;
 
 	if (!comm_cluster_rec) {
 		/* This means the addr wasn't set up already */
@@ -792,12 +730,7 @@ static int _open_controller(slurm_addr_t *addr, int *index,
 			return SLURM_ERROR;
 	}
 
-#ifdef HAVE_NATIVE_CRAY
-	max_retry_period = 180;
-#else
-	max_retry_period = slurm_conf.msg_timeout;
-#endif
-	for (retry = 0; retry < max_retry_period; retry++) {
+	for (int retry = 0; retry < slurm_conf.msg_timeout; retry++) {
 		if (retry)
 			sleep(1);
 		if (comm_cluster_rec) {
@@ -1106,7 +1039,7 @@ int slurm_receive_msg(int fd, slurm_msg_t *msg, int timeout)
 	 *  length and allocate space on the heap for a buffer containing
 	 *  the message.
 	 */
-	if (slurm_msg_recvfrom_timeout(fd, &buf, &buflen, 0, timeout) < 0) {
+	if (slurm_msg_recvfrom_timeout(fd, &buf, &buflen, timeout) < 0) {
 		rc = errno;
 		if (!rc)
 			rc = SLURMCTLD_COMMUNICATIONS_RECEIVE_ERROR;
@@ -1126,6 +1059,12 @@ int slurm_receive_msg(int fd, slurm_msg_t *msg, int timeout)
 endit:
 	slurm_seterrno(rc);
 
+	/*
+	 * We just set errno, we need to return SLURM_ERROR if not SLURM_SUCCESS
+	 */
+	if (rc != SLURM_SUCCESS)
+		rc = SLURM_ERROR;
+
 	return rc;
 }
 
@@ -1199,7 +1138,7 @@ List slurm_receive_msgs(int fd, int steps, int timeout)
 	 *  length and allocate space on the heap for a buffer containing
 	 *  the message.
 	 */
-	if (slurm_msg_recvfrom_timeout(fd, &buf, &buflen, 0, timeout) < 0) {
+	if (slurm_msg_recvfrom_timeout(fd, &buf, &buflen, timeout) < 0) {
 		forward_init(&header.forward);
 		rc = errno;
 		goto total_return;
@@ -1403,7 +1342,7 @@ extern List slurm_receive_resp_msgs(int fd, int steps, int timeout)
 	 * length and allocate space on the heap for a buffer containing the
 	 * message.
 	 */
-	if (slurm_msg_recvfrom_timeout(fd, &buf, &buflen, 0, timeout) < 0) {
+	if (slurm_msg_recvfrom_timeout(fd, &buf, &buflen, timeout) < 0) {
 		forward_init(&header.forward);
 		rc = errno;
 		goto total_return;
@@ -1558,6 +1497,10 @@ int slurm_receive_msg_and_forward(int fd, slurm_addr_t *orig_addr,
 	void *auth_cred = NULL;
 	buf_t *buffer;
 	char *peer = NULL;
+	bool keep_buffer = false;
+
+	if (msg->flags & SLURM_MSG_KEEP_BUFFER)
+		keep_buffer = true;
 
 	xassert(fd >= 0);
 
@@ -1590,7 +1533,7 @@ int slurm_receive_msg_and_forward(int fd, slurm_addr_t *orig_addr,
 	 *  length and allocate space on the heap for a buffer containing
 	 *  the message.
 	 */
-	if (slurm_msg_recvfrom_timeout(fd, &buf, &buflen, 0,
+	if (slurm_msg_recvfrom_timeout(fd, &buf, &buflen,
 				       (slurm_conf.msg_timeout * 1000)) < 0) {
 		forward_init(&header.forward);
 		rc = errno;
@@ -1724,6 +1667,8 @@ skip_auth:
 	msg->msg_type = header.msg_type;
 	msg->flags = header.flags;
 
+	msg->body_offset = get_buf_offset(buffer);
+
 	if ((header.body_length != remaining_buf(buffer)) ||
 	    _check_hash(buffer, &header, msg, auth_cred) ||
 	     (unpack_msg(msg, buffer) != SLURM_SUCCESS) ) {
@@ -1734,7 +1679,10 @@ skip_auth:
 	}
 	msg->auth_cred = auth_cred;
 
-	FREE_NULL_BUFFER(buffer);
+	if (keep_buffer)
+		msg->buffer = buffer;
+	else
+		FREE_NULL_BUFFER(buffer);
 	rc = SLURM_SUCCESS;
 
 total_return:
@@ -1906,7 +1854,6 @@ extern int slurm_send_node_msg(int fd, slurm_msg_t *msg)
 		memset(&persist_msg, 0, sizeof(persist_msg_t));
 		persist_msg.msg_type  = msg->msg_type;
 		persist_msg.data      = msg->data;
-		persist_msg.data_size = msg->data_size;
 
 		buffer = slurm_persist_msg_pack(msg->conn, &persist_msg);
 		if (!buffer)    /* pack error */
@@ -1979,7 +1926,6 @@ cleanup:
 size_t slurm_write_stream(int open_fd, char *buffer, size_t size)
 {
 	return slurm_send_timeout(open_fd, buffer, size,
-	                          SLURM_PROTOCOL_NO_SEND_RECV_FLAGS,
 	                          (slurm_conf.msg_timeout * 1000));
 }
 
@@ -1994,7 +1940,6 @@ size_t slurm_write_stream(int open_fd, char *buffer, size_t size)
 size_t slurm_read_stream(int open_fd, char *buffer, size_t size)
 {
 	return slurm_recv_timeout(open_fd, buffer, size,
-	                          SLURM_PROTOCOL_NO_SEND_RECV_FLAGS,
 	                          (slurm_conf.msg_timeout * 1000));
 }
 
@@ -2123,8 +2068,7 @@ extern void response_init(slurm_msg_t *resp_msg, slurm_msg_t *msg,
 	 * Skip sending an auth credential on the reply. Clients don't need
 	 * it, and already implicitly trust the connection.
 	 */
-	if (resp_msg->protocol_version >= SLURM_23_02_PROTOCOL_VERSION)
-		resp_msg->flags |= SLURM_NO_AUTH_CRED;
+	resp_msg->flags |= SLURM_NO_AUTH_CRED;
 }
 
 /**********************************************************************\
@@ -2188,7 +2132,9 @@ int slurm_send_rc_err_msg(slurm_msg_t *msg, int rc, char *err_msg)
  * IN msg	  - msg to respond to.
  * IN cluster_rec - cluster to direct msg to.
  */
-int slurm_send_reroute_msg(slurm_msg_t *msg, slurmdb_cluster_rec_t *cluster_rec)
+int slurm_send_reroute_msg(slurm_msg_t *msg,
+			   slurmdb_cluster_rec_t *cluster_rec,
+			   char *stepmgr)
 {
 	slurm_msg_t resp_msg;
 	reroute_msg_t reroute_msg = {0};
@@ -2200,6 +2146,7 @@ int slurm_send_reroute_msg(slurm_msg_t *msg, slurmdb_cluster_rec_t *cluster_rec)
 
 	/* Don't free the cluster_rec, it's pointing to the actual object. */
 	reroute_msg.working_cluster_rec = cluster_rec;
+	reroute_msg.stepmgr = stepmgr;
 
 	response_init(&resp_msg, msg, RESPONSE_SLURM_REROUTE_MSG,
 			&reroute_msg);
@@ -2220,7 +2167,6 @@ int slurm_send_reroute_msg(slurm_msg_t *msg, slurmdb_cluster_rec_t *cluster_rec)
 extern int slurm_send_recv_msg(int fd, slurm_msg_t *req,
 			       slurm_msg_t *resp, int timeout)
 {
-	int rc = -1;
 	slurm_msg_t_init(resp);
 
 	/* If we are using a persistent connection make sure it is the one we
@@ -2232,15 +2178,18 @@ extern int slurm_send_recv_msg(int fd, slurm_msg_t *req,
 		resp->conn = req->conn;
 	}
 
-	if (slurm_send_node_msg(fd, req) >= 0) {
-		/* no need to adjust and timeouts here since we are not
-		   forwarding or expecting anything other than 1 message
-		   and the regular timeout will be altered in
-		   slurm_receive_msg if it is 0 */
-		rc = slurm_receive_msg(fd, resp, timeout);
-	}
+	if (slurm_send_node_msg(fd, req) < 0)
+		return -1;
 
-	return rc;
+	/*
+	 * No need to adjust the timeout here since we are not forwarding or
+	 * expecting anything other than one message. The default timeout will
+	 * be used if it is set to 0.
+	 */
+	if (slurm_receive_msg(fd, resp, timeout))
+		return -1;
+
+	return 0;
 }
 
 /*
@@ -2409,25 +2358,27 @@ tryagain:
 		 */
 		verbose("RPC rate limited %d time(s). Sleeping then trying again.",
 			ratelimited);
-		sleep(1);
+		sleep(ratelimited);
 		goto tryagain;
 	}
 
 	if (!rc && (response_msg->msg_type == RESPONSE_SLURM_REROUTE_MSG)) {
 		reroute_msg_t *rr_msg = response_msg->data;
 
-		/*
-		 * Don't expect mutliple hops but in the case it does
-		 * happen, free the previous rr cluster_rec.
-		 */
-		if (comm_cluster_rec &&
-		    (comm_cluster_rec != save_comm_cluster_rec))
-			slurmdb_destroy_cluster_rec(comm_cluster_rec);
+		if (rr_msg->working_cluster_rec) {
+			/*
+			 * Don't expect mutliple hops but in the case it does
+			 * happen, free the previous rr cluster_rec.
+			 */
+			if (comm_cluster_rec &&
+			    (comm_cluster_rec != save_comm_cluster_rec))
+				slurmdb_destroy_cluster_rec(comm_cluster_rec);
 
-		comm_cluster_rec = rr_msg->working_cluster_rec;
-		slurmdb_setup_cluster_rec(comm_cluster_rec);
-		rr_msg->working_cluster_rec = NULL;
-		goto tryagain;
+			comm_cluster_rec = rr_msg->working_cluster_rec;
+			slurmdb_setup_cluster_rec(comm_cluster_rec);
+			rr_msg->working_cluster_rec = NULL;
+			goto tryagain;
+		}
 	}
 
 	if (comm_cluster_rec != save_comm_cluster_rec)
@@ -2693,7 +2644,7 @@ List slurm_send_addr_recv_msgs(slurm_msg_t *msg, char *name, int timeout)
 	List ret_list = NULL;
 	int fd = -1;
 	ret_data_info_t *ret_data_info = NULL;
-	ListIterator itr;
+	list_itr_t *itr;
 	int i;
 
 	slurm_mutex_lock(&conn_lock);
