@@ -92,7 +92,7 @@ static void _msg_handler_destruct(void *obj)
 
 void pmixp_conn_cleanup(void)
 {
-	ListIterator it = list_iterator_create(_conn_list);
+	list_itr_t *it = list_iterator_create(_conn_list);
 	pmixp_conn_t *hndl = NULL;
 	while ((hndl = list_next(it))) {
 		if (PMIXP_CONN_EMPTY == hndl->type) {
