@@ -1,7 +1,7 @@
 /*****************************************************************************\
  *  common.c - definitions for functions common to all modules in sacctmgr.
  *****************************************************************************
- *  Copyright (C) 2010-2015 SchedMD LLC.
+ *  Copyright (C) SchedMD LLC.
  *  Copyright (C) 2008 Lawrence Livermore National Security.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -467,6 +467,8 @@ static print_field_t *_get_print_field(char *object)
 		field->print_routine = print_fields_uint;
 	} else if (!xstrncasecmp("MaxTRES", object,
 				 MAX(command_len, 7)) ||
+		   !xstrncasecmp("MaxTRESPJ", object,
+				 MAX(command_len, 9)) ||
 		   !xstrncasecmp("MaxTRESPerJob", object,
 				 MAX(command_len, 11))) {
 		field->type = PRINT_MAXT;
@@ -474,13 +476,17 @@ static print_field_t *_get_print_field(char *object)
 		field->len = 13;
 		field->print_routine = sacctmgr_print_tres;
 	} else if (!xstrncasecmp("MaxTRESPerNode", object,
-				 MAX(command_len, 11))) {
+				 MAX(command_len, 11)) ||
+		   !xstrncasecmp("MaxTRESPN", object,
+				 MAX(command_len, 9))) {
 		field->type = PRINT_MAXTN;
 		field->name = xstrdup("MaxTRESPerNode");
 		field->len = 14;
 		field->print_routine = sacctmgr_print_tres;
 	} else if (!xstrncasecmp("MaxTRESMinsPerJob", object,
-				 MAX(command_len, 8))) {
+				 MAX(command_len, 8)) ||
+		   !xstrncasecmp("MaxTRESMinsPJ", object,
+				 MAX(command_len, 13))) {
 		field->type = PRINT_MAXTM;
 		field->name = xstrdup("MaxTRESMins");
 		field->len = 13;
@@ -493,7 +499,7 @@ static print_field_t *_get_print_field(char *object)
 				 MAX(command_len, 15))) {
 		field->type = PRINT_MAXTRMA;
 		field->name = xstrdup("MaxTRESRunMinsPA");
-		field->len = 15;
+		field->len = 16;
 		field->print_routine = sacctmgr_print_tres;
 	} else if (!xstrncasecmp("MaxTRESRunMinsPerUser", object,
 				 MAX(command_len, 8)) ||
@@ -501,7 +507,7 @@ static print_field_t *_get_print_field(char *object)
 				 MAX(command_len, 8))) {
 		field->type = PRINT_MAXTRM;
 		field->name = xstrdup("MaxTRESRunMinsPU");
-		field->len = 15;
+		field->len = 16;
 		field->print_routine = sacctmgr_print_tres;
 	} else if (!xstrncasecmp("MaxTRESPerAccount", object,
 				 MAX(command_len, 11)) ||
@@ -908,9 +914,9 @@ extern int sacctmgr_remove_assoc_usage(slurmdb_assoc_cond_t *assoc_cond)
 	List update_list = NULL;
 	List local_assoc_list = NULL;
 	List local_cluster_list = NULL;
-	ListIterator itr = NULL;
-	ListIterator itr2 = NULL;
-	ListIterator itr3 = NULL;
+	list_itr_t *itr = NULL;
+	list_itr_t *itr2 = NULL;
+	list_itr_t *itr3 = NULL;
 	char *account = NULL;
 	char *cluster = NULL;
 	char *user = NULL;
@@ -1042,7 +1048,7 @@ extern int sacctmgr_update_qos_usage(slurmdb_qos_cond_t *qos_cond,
 	List cluster_list;
 	List local_qos_list = NULL;
 	List local_cluster_list = NULL;
-	ListIterator itr = NULL, itr2 = NULL;
+	list_itr_t *itr = NULL, *itr2 = NULL;
 	char *qos_name = NULL, *cluster_name = NULL;
 	slurmdb_qos_rec_t* rec = NULL;
 	slurmdb_cluster_rec_t* cluster_rec = NULL;
@@ -1255,7 +1261,7 @@ extern slurmdb_assoc_rec_t *sacctmgr_find_assoc_from_list(
 	List assoc_list, char *user, char *account,
 	char *cluster, char *partition)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	slurmdb_assoc_rec_t * assoc = NULL;
 
 	if (!assoc_list)
@@ -1291,7 +1297,7 @@ extern slurmdb_assoc_rec_t *sacctmgr_find_assoc_from_list(
 extern slurmdb_assoc_rec_t *sacctmgr_find_account_base_assoc_from_list(
 	List assoc_list, char *account, char *cluster)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	slurmdb_assoc_rec_t *assoc = NULL;
 	char *temp = "root";
 
@@ -1320,7 +1326,7 @@ extern slurmdb_assoc_rec_t *sacctmgr_find_account_base_assoc_from_list(
 extern slurmdb_qos_rec_t *sacctmgr_find_qos_from_list(
 	List qos_list, char *name)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	slurmdb_qos_rec_t *qos = NULL;
 	char *working_name = NULL;
 
@@ -1346,7 +1352,7 @@ extern slurmdb_qos_rec_t *sacctmgr_find_qos_from_list(
 extern slurmdb_res_rec_t *sacctmgr_find_res_from_list(
 	List res_list, uint32_t id, char *name, char *server)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	slurmdb_res_rec_t *res = NULL;
 
 	if ((id == NO_VAL) && (!name || !res_list))
@@ -1368,7 +1374,7 @@ extern slurmdb_res_rec_t *sacctmgr_find_res_from_list(
 extern slurmdb_user_rec_t *sacctmgr_find_user_from_list(
 	List user_list, char *name)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	slurmdb_user_rec_t *user = NULL;
 
 	if (!name || !user_list)
@@ -1388,7 +1394,7 @@ extern slurmdb_user_rec_t *sacctmgr_find_user_from_list(
 extern slurmdb_account_rec_t *sacctmgr_find_account_from_list(
 	List acct_list, char *name)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	slurmdb_account_rec_t *account = NULL;
 
 	if (!name || !acct_list)
@@ -1408,7 +1414,7 @@ extern slurmdb_account_rec_t *sacctmgr_find_account_from_list(
 extern slurmdb_cluster_rec_t *sacctmgr_find_cluster_from_list(
 	List cluster_list, char *name)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	slurmdb_cluster_rec_t *cluster = NULL;
 
 	if (!name || !cluster_list)
@@ -1427,7 +1433,7 @@ extern slurmdb_cluster_rec_t *sacctmgr_find_cluster_from_list(
 extern slurmdb_wckey_rec_t *sacctmgr_find_wckey_from_list(
 	List wckey_list, char *user, char *name, char *cluster)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	slurmdb_wckey_rec_t * wckey = NULL;
 
 	if (!wckey_list)
@@ -1588,7 +1594,7 @@ extern void sacctmgr_print_coord_list(
 	print_field_t *field, void *input, int last)
 {
 	int abs_len = abs(field->len);
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	char *print_this = NULL;
 	List value = NULL;
 	slurmdb_coord_rec_t *object = NULL;
@@ -1887,7 +1893,7 @@ extern void sacctmgr_print_federation(slurmdb_federation_rec_t *fed)
 		xfree(tmp_flags);
 	}
 	if (fed->cluster_list) {
-		ListIterator itr = list_iterator_create(fed->cluster_list);
+		list_itr_t *itr = list_iterator_create(fed->cluster_list);
 		slurmdb_cluster_rec_t *cluster = NULL;
 		while ((cluster = list_next(itr))) {
 			char *tmp_name = cluster->name;
@@ -2156,7 +2162,7 @@ extern int sort_coord_list(void *a, void *b)
 extern List sacctmgr_process_format_list(List format_list)
 {
 	List print_fields_list = list_create(destroy_print_field);
-	ListIterator itr = list_iterator_create(format_list);
+	list_itr_t *itr = list_iterator_create(format_list);
 	print_field_t *field = NULL;
 	char *object = NULL;
 
@@ -2176,7 +2182,7 @@ extern int sacctmgr_validate_cluster_list(List cluster_list)
 	List temp_list = NULL;
 	char *cluster = NULL;
 	int rc = SLURM_SUCCESS;
-	ListIterator itr = NULL, itr_c = NULL;
+	list_itr_t *itr = NULL, *itr_c = NULL;
 
 	xassert(cluster_list);
 
