@@ -2,7 +2,7 @@
  *  qos_functions.c - functions dealing with qoss in the
  *                        accounting system.
  *****************************************************************************
- *  Copyright (C) 2010-2015 SchedMD LLC.
+ *  Copyright (C) SchedMD LLC.
  *  Copyright (C) 2002-2008 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Danny Auble <da@llnl.gov>
@@ -145,7 +145,7 @@ static int _set_cond(int *start, int argc, char **argv,
 			if (format_list)
 				slurm_addto_char_list(format_list, argv[i]+end);
 		} else if (!xstrncasecmp(argv[i], "Ids", MAX(command_len, 1))) {
-			ListIterator itr = NULL;
+			list_itr_t *itr = NULL;
 			char *temp = NULL;
 			uint32_t id = 0;
 
@@ -510,8 +510,12 @@ static int _set_rec(int *start, int argc, char **argv,
 				xfree(tmp_char);
 			} else
 				exit_code = 1;
-		} else if (!xstrncasecmp(argv[i], "MaxTRESPerJob",
-					 MAX(command_len, 7))) {
+		} else if (!xstrncasecmp(argv[i], "MaxTRES",
+					 MAX(command_len, 7)) ||
+		           !xstrncasecmp(argv[i], "MaxTRESPJ",
+					 MAX(command_len, 9)) ||
+		           !xstrncasecmp(argv[i], "MaxTRESPerJob",
+					 MAX(command_len, 11))) {
 			sacctmgr_initialize_g_tres_list();
 
 			if ((tmp_char = slurmdb_format_tres_str(
@@ -524,7 +528,9 @@ static int _set_rec(int *start, int argc, char **argv,
 			} else
 				exit_code = 1;
 		} else if (!xstrncasecmp(argv[i], "MaxTRESPerNode",
-					 MAX(command_len, 11))) {
+					 MAX(command_len, 11)) ||
+			   !xstrncasecmp(argv[i], "MaxTRESPN",
+					 MAX(command_len, 9))) {
 			sacctmgr_initialize_g_tres_list();
 
 			if ((tmp_char = slurmdb_format_tres_str(
@@ -552,7 +558,9 @@ static int _set_rec(int *start, int argc, char **argv,
 			} else
 				exit_code = 1;
 		} else if (!xstrncasecmp(argv[i], "MaxTRESMinsPerJob",
-					 MAX(command_len, 8))) {
+					 MAX(command_len, 8)) ||
+			   !xstrncasecmp(argv[i], "MaxTRESMinsPJ",
+					 MAX(command_len, 13))) {
 			sacctmgr_initialize_g_tres_list();
 
 			if ((tmp_char = slurmdb_format_tres_str(
@@ -564,7 +572,11 @@ static int _set_rec(int *start, int argc, char **argv,
 				xfree(tmp_char);
 			} else
 				exit_code = 1;
-		} else if (!xstrncasecmp(argv[i], "MaxTRESRunMinsPA",
+		} else if (!xstrncasecmp(argv[i], "MaxTRESRunMinsPerAccount",
+					 MAX(command_len, 24)) ||
+			   !xstrncasecmp(argv[i], "MaxTRESRunMinsPerAcct",
+					 MAX(command_len, 20)) ||
+			   !xstrncasecmp(argv[i], "MaxTRESRunMinsPA",
 					 MAX(command_len, 16))) {
 			sacctmgr_initialize_g_tres_list();
 
@@ -577,8 +589,10 @@ static int _set_rec(int *start, int argc, char **argv,
 				xfree(tmp_char);
 			} else
 				exit_code = 1;
-		} else if (!xstrncasecmp(argv[i], "MaxTRESRunMinsPU",
-					 MAX(command_len, 8))) {
+		} else if (!xstrncasecmp(argv[i], "MaxTRESRunMinsPerUser",
+					 MAX(command_len, 20)) ||
+			   !xstrncasecmp(argv[i], "MaxTRESRunMinsPU",
+					 MAX(command_len, 16))) {
 			sacctmgr_initialize_g_tres_list();
 
 			if ((tmp_char = slurmdb_format_tres_str(
@@ -718,7 +732,7 @@ static bool _isdefault(List qos_list)
 	int rc = 0;
 	slurmdb_assoc_cond_t assoc_cond;
 	slurmdb_assoc_rec_t *assoc = NULL;
-	ListIterator itr;
+	list_itr_t *itr;
 	List ret_list = NULL;
 	char *name = NULL;
 
@@ -787,7 +801,7 @@ extern int sacctmgr_add_qos(int argc, char **argv)
 {
 	int rc = SLURM_SUCCESS;
 	int i;
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	slurmdb_qos_rec_t *qos = NULL;
 	slurmdb_qos_rec_t *start_qos = xmalloc(sizeof(slurmdb_qos_rec_t));
 	List name_list = list_create(xfree_ptr);
@@ -912,8 +926,8 @@ extern int sacctmgr_list_qos(int argc, char **argv)
 	int rc = SLURM_SUCCESS;
 	slurmdb_qos_cond_t *qos_cond = xmalloc(sizeof(slurmdb_qos_cond_t));
  	int i=0;
-	ListIterator itr = NULL;
-	ListIterator itr2 = NULL;
+	list_itr_t *itr = NULL;
+	list_itr_t *itr2 = NULL;
 	slurmdb_qos_rec_t *qos = NULL;
 	List qos_list = NULL;
 	int field_count = 0;
@@ -949,6 +963,8 @@ extern int sacctmgr_list_qos(int argc, char **argv)
 				      "MaxJobsPerUser,"
 				      "MaxSubmitJobsPerUser,"
 				      "MaxTRESPerAcct,"
+				      "MaxTRESRunMinsPerAcct%22,"
+				      "MaxTRESRunMinsPerUser%22,"
 				      "MaxJobsPerAcct,"
 				      "MaxSubmitJobsPerAcct,MinTRES");
 	}
@@ -1361,7 +1377,7 @@ extern int sacctmgr_modify_qos(int argc, char **argv)
 	ret_list = slurmdb_qos_modify(db_conn, qos_cond, qos);
 	if (ret_list && list_count(ret_list)) {
 		char *object = NULL;
-		ListIterator itr = list_iterator_create(ret_list);
+		list_itr_t *itr = list_iterator_create(ret_list);
 		printf(" Modified qos...\n");
 		while((object = list_next(itr))) {
 			printf("  %s\n", object);
@@ -1436,7 +1452,7 @@ extern int sacctmgr_delete_qos(int argc, char **argv)
 
 	if (ret_list && list_count(ret_list)) {
 		char *object = NULL;
-		ListIterator itr = NULL;
+		list_itr_t *itr = NULL;
 
 		/* Check to see if person is trying to remove a default
 		 * qos of an association.  _isdefault only works with the
