@@ -48,12 +48,15 @@
 
 #include "src/common/assoc_mgr.h"
 #include "src/common/cpu_frequency.h"
+#include "src/common/forward.h"
+#include "src/common/macros.h"
 #include "src/common/run_command.h"
-#include "src/interfaces/topology.h"
 #include "src/common/setproctitle.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_protocol_pack.h"
 #include "src/common/slurm_rlimits_info.h"
+#include "src/common/macros.h"
+#include "src/common/port_mgr.h"
 #include "src/common/spank.h"
 #include "src/common/stepd_api.h"
 #include "src/common/xmalloc.h"
@@ -64,16 +67,18 @@
 #include "src/interfaces/acct_gather_profile.h"
 #include "src/interfaces/auth.h"
 #include "src/interfaces/cgroup.h"
-#include "src/interfaces/core_spec.h"
 #include "src/interfaces/gpu.h"
 #include "src/interfaces/gres.h"
 #include "src/interfaces/hash.h"
 #include "src/interfaces/job_container.h"
 #include "src/interfaces/jobacct_gather.h"
 #include "src/interfaces/mpi.h"
+#include "src/interfaces/prep.h"
 #include "src/interfaces/proctrack.h"
+#include "src/interfaces/select.h"
 #include "src/interfaces/switch.h"
 #include "src/interfaces/task.h"
+#include "src/interfaces/topology.h"
 
 #include "src/slurmd/common/privileges.h"
 #include "src/slurmd/common/set_oomadj.h"
@@ -85,6 +90,8 @@
 #include "src/slurmd/slurmstepd/slurmstepd.h"
 #include "src/slurmd/slurmstepd/slurmstepd_job.h"
 
+#include "src/stepmgr/stepmgr.h"
+
 static int _init_from_slurmd(int sock, char **argv, slurm_addr_t **_cli,
 			    slurm_msg_t **_msg);
 
@@ -113,6 +120,182 @@ int slurmstepd_blocked_signals[] = {
 slurmd_conf_t * conf;
 extern char  ** environ;
 
+list_t *job_list = NULL;
+job_record_t *job_step_ptr = NULL;
+list_t *job_node_array = NULL;
+time_t last_job_update = 0;
+bool time_limit_thread_shutdown = false;
+pthread_t time_limit_thread_id = 0;
+
+static int _foreach_ret_data_info(void *x, void *arg)
+{
+	int rc;
+	ret_data_info_t *ret_data_info = x;
+
+	if ((rc = slurm_get_return_code(ret_data_info->type,
+					ret_data_info->data))) {
+		error("stepmgr failed to send message %s: rc=%d(%s)",
+		      rpc_num2string(ret_data_info->type), rc,
+		      slurm_strerror(rc));
+		return SLURM_ERROR;
+	}
+
+	return SLURM_SUCCESS;
+}
+
+static void *_rpc_thread(void *data)
+{
+	agent_arg_t *agent_arg_ptr = data;
+	slurm_msg_t msg;
+	slurm_msg_t_init(&msg);
+
+	msg.data = agent_arg_ptr->msg_args;
+	msg.flags = agent_arg_ptr->msg_flags;
+	msg.msg_type = agent_arg_ptr->msg_type;
+	msg.protocol_version = agent_arg_ptr->protocol_version;
+
+	slurm_msg_set_r_uid(&msg, agent_arg_ptr->r_uid);
+
+	if (agent_arg_ptr->addr) {
+		msg.address = *agent_arg_ptr->addr;
+		if (slurm_send_only_node_msg(&msg)) {
+			error("failed to send message type %d/%s",
+			      msg.msg_type, rpc_num2string(msg.msg_type));
+		}
+	} else {
+		list_t *ret_list = NULL;
+		if (!(ret_list = start_msg_tree(agent_arg_ptr->hostlist,
+						&msg, 0))) {
+			error("%s: no ret_list given", __func__);
+		} else {
+			list_for_each(ret_list, _foreach_ret_data_info, NULL);
+			FREE_NULL_LIST(ret_list);
+		}
+	}
+
+	purge_agent_args(agent_arg_ptr);
+
+	return NULL;
+}
+
+static void _agent_queue_request(agent_arg_t *agent_arg_ptr)
+{
+	slurm_thread_create_detached(_rpc_thread, agent_arg_ptr);
+}
+
+extern job_record_t *find_job_record(uint32_t job_id)
+{
+	xassert(job_step_ptr);
+
+	return job_step_ptr;
+}
+
+static void *_step_time_limit_thread(void *data)
+{
+	time_t now;
+
+	xassert(job_step_ptr);
+
+	while (!time_limit_thread_shutdown) {
+		now = time(NULL);
+		slurm_mutex_lock(&stepmgr_mutex);
+		list_for_each(job_step_ptr->step_list,
+			      check_job_step_time_limit, &now);
+		slurm_mutex_unlock(&stepmgr_mutex);
+		sleep(1);
+	}
+
+	return NULL;
+}
+
+stepmgr_ops_t stepd_stepmgr_ops = {
+	.find_job_record = find_job_record,
+	.last_job_update = &last_job_update,
+	.agent_queue_request = _agent_queue_request
+};
+
+static int _foreach_job_node_array(void *x, void *arg)
+{
+	node_record_t *job_node_ptr = x;
+	int *table_index = arg;
+
+	config_record_t *config_ptr;
+	config_ptr = create_config_record();
+	config_ptr->boards = job_node_ptr->boards;
+	config_ptr->core_spec_cnt = job_node_ptr->core_spec_cnt;
+	config_ptr->cores = job_node_ptr->cores;
+	config_ptr->cpu_spec_list = xstrdup(job_node_ptr->cpu_spec_list);
+	config_ptr->cpus = job_node_ptr->cpus;
+	config_ptr->feature = xstrdup(job_node_ptr->features);
+	config_ptr->gres = xstrdup(job_node_ptr->gres);
+	config_ptr->node_bitmap = bit_alloc(node_record_count);
+	config_ptr->nodes = xstrdup(job_node_ptr->name);
+	config_ptr->real_memory = job_node_ptr->real_memory;
+	config_ptr->res_cores_per_gpu = job_node_ptr->res_cores_per_gpu;
+	config_ptr->threads = job_node_ptr->threads;
+	config_ptr->tmp_disk = job_node_ptr->tmp_disk;
+	config_ptr->tot_sockets = job_node_ptr->tot_sockets;
+	config_ptr->weight = job_node_ptr->weight;
+
+	*table_index = bit_ffs_from_bit(job_step_ptr->node_bitmap, *table_index);
+
+	job_node_ptr->config_ptr = config_ptr;
+	insert_node_record_at(job_node_ptr, *table_index);
+
+	(*table_index)++;
+
+	/*
+	 * Sanity check to make sure we can take a version we
+	 * actually understand.
+	 */
+	if (job_node_ptr->protocol_version < SLURM_MIN_PROTOCOL_VERSION)
+		job_node_ptr->protocol_version = SLURM_MIN_PROTOCOL_VERSION;
+
+	return SLURM_SUCCESS;
+}
+
+static void _setup_stepmgr_nodes(void)
+{
+	int table_index = 0;
+	init_node_conf();
+
+	xassert(job_node_array);
+	/*
+	 * next_node_bitmap() asserts
+	 * bit_size(node_bitmap) == node_record_count
+	 */
+	node_record_count = bit_size(job_step_ptr->node_bitmap);
+	grow_node_record_table_ptr();
+	list_for_each(job_node_array, _foreach_job_node_array, &table_index);
+}
+
+static void _init_stepd_stepmgr(void)
+{
+	if (!job_step_ptr)
+		return;
+
+	stepd_stepmgr_ops.up_node_bitmap =
+		bit_alloc(bit_size(job_step_ptr->node_bitmap));
+	bit_set_all(stepd_stepmgr_ops.up_node_bitmap);
+	stepmgr_init(&stepd_stepmgr_ops);
+	reserve_port_stepmgr_init(job_step_ptr);
+
+	_setup_stepmgr_nodes();
+
+	if (!xstrcasecmp(slurm_conf.accounting_storage_type,
+			 "accounting_storage/slurmdbd")) {
+		xfree(slurm_conf.accounting_storage_type);
+		slurm_conf.accounting_storage_type =
+			xstrdup("accounting_storage/ctld_relay");
+		acct_storage_g_init();
+	} else {
+			acct_storage_g_init();
+	}
+
+	slurm_thread_create(&time_limit_thread_id, _step_time_limit_thread,
+			    NULL);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -145,6 +328,8 @@ main (int argc, char **argv)
 		goto ending;
 	}
 
+	_init_stepd_stepmgr();
+
 	/* fork handlers cause mutexes on some global data structures
 	 * to be re-initialized after the fork. */
 	slurm_conf_install_fork_handlers();
@@ -187,6 +372,8 @@ ending:
 extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *step,
 			 slurm_addr_t *cli, int rc, bool only_mem)
 {
+	time_limit_thread_shutdown = true;
+
 	slurm_mutex_lock(&cleanup_mutex);
 
 	if (cleanup)
@@ -204,7 +391,7 @@ extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *step,
 		/* signal the message thread to shutdown, and wait for it */
 		if (step->msg_handle)
 			eio_signal_shutdown(step->msg_handle);
-		pthread_join(step->msgid, NULL);
+		slurm_thread_join(step->msgid);
 	}
 
 	mpi_fini();
@@ -224,22 +411,26 @@ extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *step,
 	run_command_shutdown();
 
 	if (step->step_id.step_id == SLURM_EXTERN_CONT) {
-		uint32_t jobid;
-#ifdef HAVE_NATIVE_CRAY
-		if (step->het_job_id && (step->het_job_id != NO_VAL))
-			jobid = step->het_job_id;
-		else
-			jobid = step->step_id.job_id;
-#else
-		jobid = step->step_id.job_id;
-#endif
-		if (container_g_stepd_delete(jobid))
-			error("container_g_stepd_delete(%u): %m", jobid);
+		if (container_g_stepd_delete(step->step_id.job_id))
+			error("container_g_stepd_delete(%u): %m",
+			      step->step_id.job_id);
 	}
 
+	/*
+	 * join() must be done before _step_cleanup() where job_step_ptr is
+	 * freed.
+	 */
+	slurm_thread_join(time_limit_thread_id);
+
 #ifdef MEMORY_LEAK_DEBUG
 	acct_gather_conf_destroy();
-	(void) core_spec_g_fini();
+	acct_storage_g_fini();
+
+	if (job_step_ptr) {
+		xfree(job_step_ptr->resv_ports);
+		reserve_port_stepmgr_init(job_step_ptr);
+	}
+
 	_step_cleanup(step, msg, rc);
 
 	fini_setproctitle();
@@ -249,6 +440,7 @@ extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *step,
 	xfree(cli);
 	xfree(conf->block_map);
 	xfree(conf->block_map_inv);
+	xfree(conf->conffile);
 	xfree(conf->hostname);
 	xfree(conf->hwloc_xml);
 	xfree(conf->logfile);
@@ -256,6 +448,7 @@ extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *step,
 	xfree(conf->node_topo_addr);
 	xfree(conf->node_topo_pattern);
 	xfree(conf->spooldir);
+	xfree(conf->stepd_loc);
 	xfree(conf->cpu_spec_list);
 	xfree(conf);
 #endif
@@ -539,7 +732,9 @@ static void _set_job_log_prefix(slurm_step_id_t *step_id)
 
 	log_build_step_id_str(step_id, tmp_char, sizeof(tmp_char),
 			      STEP_ID_FLAG_NO_PREFIX);
-	buf = xstrdup_printf("[%s]", tmp_char);
+	buf = xstrdup_printf("[%s%s]",
+			     tmp_char,
+			     job_step_ptr ? " stepmgr" : "");
 
 	setproctitle("%s", buf);
 	/* note: will claim ownership of buf, do not free */
@@ -572,6 +767,11 @@ _init_from_slurmd(int sock, char **argv, slurm_addr_t **_cli,
 	if (!(conf = _read_slurmd_conf_lite(sock)))
 		fatal("Failed to read conf from slurmd");
 
+	/*
+	 * Init select plugin after reading slurm.conf and before receiving step
+	 */
+	select_g_init(false);
+
 	slurm_conf.slurmd_port = conf->port;
 	slurm_conf.slurmd_syslog_debug = conf->syslog_debug;
 	/*
@@ -653,8 +853,8 @@ _init_from_slurmd(int sock, char **argv, slurm_addr_t **_cli,
 	}
 
 	/* Init switch before unpack_msg to only init the default */
-	if (switch_init(1) != SLURM_SUCCESS)
-		fatal( "failed to initialize authentication plugin" );
+	if (switch_g_init(true) != SLURM_SUCCESS)
+		fatal("failed to initialize switch plugin");
 
 	if (cred_g_init() != SLURM_SUCCESS)
 		fatal("failed to initialize credential plugin");
@@ -673,10 +873,22 @@ _init_from_slurmd(int sock, char **argv, slurm_addr_t **_cli,
 		step_id.step_het_comp = NO_VAL;
 		break;
 	case LAUNCH_TASKS:
-		memcpy(&step_id,
-		       &((launch_tasks_request_msg_t *)msg->data)->step_id,
-		       sizeof(step_id));
+	{
+		launch_tasks_request_msg_t *task_msg;
+		task_msg = (launch_tasks_request_msg_t *)msg->data;
+
+		memcpy(&step_id, &task_msg->step_id, sizeof(step_id));
+
+		if (task_msg->job_ptr &&
+		    !xstrcmp(conf->node_name, task_msg->job_ptr->batch_host)) {
+			/* only allow one stepd to be stepmgr. */
+			job_step_ptr = task_msg->job_ptr;
+			job_step_ptr->part_ptr = task_msg->part_ptr;
+			job_node_array = task_msg->job_node_array;
+		}
+
 		break;
+	}
 	default:
 		fatal("%s: Unrecognized launch RPC (%d)", __func__, step_type);
 		break;
@@ -691,7 +903,7 @@ _init_from_slurmd(int sock, char **argv, slurm_addr_t **_cli,
 	    (cgroup_g_init() != SLURM_SUCCESS) ||
 	    (hash_g_init() != SLURM_SUCCESS) ||
 	    (acct_gather_conf_init() != SLURM_SUCCESS) ||
-	    (core_spec_g_init() != SLURM_SUCCESS) ||
+	    (prep_g_init(NULL) != SLURM_SUCCESS) ||
 	    (proctrack_g_init() != SLURM_SUCCESS) ||
 	    (slurmd_task_init() != SLURM_SUCCESS) ||
 	    (jobacct_gather_init() != SLURM_SUCCESS) ||
@@ -834,7 +1046,7 @@ static stepd_step_rec_t *_step_setup(slurm_addr_t *cli, slurm_msg_t *msg)
 	env_array_overwrite(&step->env,"SLURM_TOPOLOGY_ADDR_PATTERN",
 			    conf->node_topo_pattern);
 
-	/* Reset addrs for dyanmic/cloud nodes to hash tables*/
+	/* Reset addrs for dynamic/cloud nodes to hash tables */
 	if (step->node_addrs &&
 	    add_remote_nodes_to_conf_tbls(step->node_list, step->node_addrs)) {
 		error("%s: failed to add node addrs: %s", __func__,
