@@ -61,6 +61,7 @@
 #include "src/interfaces/acct_gather_profile.h"
 #include "src/interfaces/gres.h"
 #include "src/interfaces/jobacct_gather.h"
+#include "src/interfaces/switch.h"
 
 #include "src/slurmd/common/fname.h"
 #include "src/slurmd/slurmd/slurmd.h"
@@ -248,6 +249,10 @@ static void _slurm_cred_to_step_rec(slurm_cred_t *cred, stepd_step_rec_t *step)
 	step->alias_list = xstrdup(cred_arg->job_alias_list);
 	step->node_list = xstrdup(cred_arg->job_hostlist);
 
+	if (cred_arg->switch_step)
+		switch_g_duplicate_stepinfo(cred_arg->switch_step,
+					    &step->switch_step);
+
 	slurm_cred_unlock_args(cred);
 }
 
@@ -454,7 +459,6 @@ extern stepd_step_rec_t *stepd_step_rec_create(launch_tasks_request_msg_t *msg,
 
 	step->timelimit   = (time_t) -1;
 	step->flags       = msg->flags;
-	step->switch_step = msg->switch_step;
 	step->open_mode   = msg->open_mode;
 	step->options     = msg->options;
 
