@@ -1,8 +1,7 @@
 /*****************************************************************************\
  *  diag.c - Slurm REST API accounting diag http operations handlers
  *****************************************************************************
- *  Copyright (C) 2020 SchedMD LLC.
- *  Written by Nathan Rini <nate@schedmd.com>
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -34,8 +33,6 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
 
-#include "config.h"
-
 #include <stdint.h>
 
 #include "slurm/slurm.h"
@@ -52,7 +49,7 @@
 #include "src/common/xstring.h"
 
 #include "src/slurmrestd/operations.h"
-#include "src/slurmrestd/plugins/openapi/dbv0.0.38/api.h"
+#include "src/slurmrestd/plugins/openapi/dbv0.0.39/api.h"
 
 /* based on sacctmgr_list_stats() */
 static int _op_handler_diag(const char *context_id,
@@ -60,32 +57,29 @@ static int _op_handler_diag(const char *context_id,
 			    data_t *query, int tag, data_t *resp, void *auth,
 			    data_parser_t *parser)
 {
-	data_t *errors = populate_response_format(resp);
-	parser_env_t penv = { 0 };
+	int rc;
 	slurmdb_stats_rec_t *stats_rec = NULL;
-	int rc = SLURM_SUCCESS;
-	void *db_conn;
+	ctxt_t *ctxt = init_connection(context_id, method, parameters, query,
+				       tag, resp, auth);
+	data_t *dstats = data_key_set(ctxt->resp, "statistics");
 
-	debug4("%s:[%s] diag handler called", __func__, context_id);
+	debug4("%s: [%s] diag handler called", __func__, context_id);
 
-	if (!(db_conn = openapi_get_db_conn(auth)))
-		resp_error(errors, ESLURM_DB_CONNECTION_INVALID, NULL,
-			   "openapi_get_db_conn");
-	else if ((rc = slurmdb_get_stats(db_conn, &stats_rec)))
-		resp_error(errors, rc, NULL, "slurmdb_get_stats");
-	else
-		rc = dump(PARSE_STATS_REC, stats_rec,
-			  data_set_dict(data_key_set(resp, "statistics")),
-			  &penv);
+	if (ctxt->rc)
+		goto cleanup;
+	else if ((rc = slurmdb_get_stats(ctxt->db_conn, &stats_rec)))
+		resp_error(ctxt, rc, "slurmdb_get_stats", "stats query failed");
 
+	DATA_DUMP(ctxt->parser, STATS_REC_PTR, stats_rec, dstats);
+
+cleanup:
 	slurmdb_destroy_stats_rec(stats_rec);
-
-	return rc;
+	return fini_connection(ctxt);
 }
 
 extern void init_op_diag(void)
 {
-	bind_operation_handler("/slurmdb/v0.0.38/diag/", _op_handler_diag, 0);
+	bind_operation_handler("/slurmdb/v0.0.39/diag/", _op_handler_diag, 0);
 }
 
 extern void destroy_op_diag(void)
